import { FrameButton, Frame, SupportedParsingSpecification, getFrame, TransactionTargetResponse } from 'frames.js';
import { Dispatch } from 'react';
import { ParseResult } from 'frames.js/frame-parsers';
import { CastId, FrameActionMessage } from '@farcaster/core';

type FarcasterSignerState = SignerStateInstance<FarcasterSigner | null>;
type FarcasterSignerPendingApproval = {
    status: "pending_approval";
    deadline: number;
    privateKey: string;
    publicKey: string;
    requestFid: number;
    requestSigner: string;
    signature: string;
    signerApprovalUrl: string;
    token: string;
};
type FarcasterSignerImpersonating = {
    status: "impersonating";
    fid: number;
    privateKey: string;
    publicKey: string;
};
type FarcasterSignerApproved = {
    status: "approved";
    fid: number;
    privateKey: string;
    publicKey: string;
};
type FarcasterSigner = FarcasterSignerPendingApproval | FarcasterSignerImpersonating | FarcasterSignerApproved;
declare const mockFarcasterSigner: FarcasterSigner;

type FarcasterFrameContext = {
    /** Connected address of user, only sent with transaction data request */
    address?: `0x${string}`;
    castId: {
        hash: `0x${string}`;
        fid: number;
    };
};
/** Creates a frame action for use with `useFrame` and a proxy */
declare const signFrameAction: ({ buttonIndex, frameContext, frameButton, signer, target, inputText, state, transactionId, url, }: {
    target?: string | undefined;
    frameButton: FrameButton;
    buttonIndex: number;
    url: string;
    signer: FarcasterSignerState["signer"];
    inputText?: string | undefined;
    state?: string | undefined;
    transactionId?: `0x${string}` | undefined;
    frameContext: FarcasterFrameContext;
}) => Promise<{
    body: FrameActionBodyPayload;
    searchParams: URLSearchParams;
}>;
declare function createFrameActionMessageWithSignerKey(signerKey: string, { fid, url, buttonIndex, castId, inputText, state, address, transactionId, }: {
    fid: number;
    url: Uint8Array;
    buttonIndex: number;
    inputText: Uint8Array | undefined;
    castId: CastId;
    state: Uint8Array | undefined;
    address: Uint8Array | undefined;
    transactionId: Uint8Array | undefined;
}): Promise<{
    message: null;
    trustedBytes: null;
} | {
    message: FrameActionMessage | null;
    trustedBytes: string;
}>;

type OnTransactionFunc = (t: OnTransactionArgs) => Promise<`0x${string}` | null>;
type UseFrameReturn<SignerStorageType = object, FrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, FrameContextType extends FrameContext = FarcasterFrameContext> = {
    /** skip frame signing, for frames that don't verify signatures */
    dangerousSkipSigning?: boolean;
    /** the route used to POST frame actions. The post_url will be added as a the `url` query parameter */
    frameActionProxy: string;
    /** the route used to GET the initial frame via proxy */
    frameGetProxy: string;
    /** an signer state object used to determine what actions are possible */
    signerState: SignerStateInstance<SignerStorageType, FrameActionBodyType, FrameContextType>;
    /** the url of the homeframe, if null / undefined won't load a frame */
    homeframeUrl: string | null | undefined;
    /** the initial frame. if not specified will fetch it from the homeframeUrl prop */
    frame?: Frame | ParseResult;
    /** connected wallet address of the user */
    connectedAddress: `0x${string}` | undefined;
    /** a function to handle mint buttons */
    onMint?: (t: OnMintArgs) => void;
    /** a function to handle transaction buttons, returns the transaction hash or null */
    onTransaction?: OnTransactionFunc;
    /** the context of this frame, used for generating Frame Action payloads */
    frameContext: FrameContextType;
    /**
     * Extra data appended to the frame action payload
     */
    extraButtonRequestPayload?: Record<string, unknown>;
    /**
     * Which specification to use for parsing the frame action payload
     *
     * @defaultValue 'farcaster'
     */
    specification?: SupportedParsingSpecification;
};
interface SignerStateInstance<SignerStorageType = object, FrameActionBodyType extends FrameActionBodyPayload = FrameActionBodyPayload, FrameContextType extends FrameContext = FarcasterFrameContext> {
    signer?: SignerStorageType | null;
    hasSigner: boolean;
    signFrameAction: (actionContext: {
        target?: string;
        frameButton: FrameButton;
        buttonIndex: number;
        url: string;
        inputText?: string;
        signer: SignerStorageType | null;
        state?: string;
        transactionId?: `0x${string}`;
        address?: `0x${string}`;
        /** Transacting address is not included in non-transaction frame actions */
        frameContext: FrameContextType | Omit<FrameContextType, "address">;
    }) => Promise<{
        body: FrameActionBodyType;
        searchParams: URLSearchParams;
    }>;
    /** isLoading frame */
    isLoading?: null | FrameStackPending;
    /** is loading the signer */
    isLoadingSigner?: boolean;
    /** A function called when a frame button is clicked without a signer */
    onSignerlessFramePress: () => void;
    logout?: () => void;
}
type FrameGETRequest = {
    method: "GET";
    url: string;
};
type FramePOSTRequest = {
    method: "POST";
    request: {
        body: object;
        searchParams: URLSearchParams;
    };
    url: string;
    /**
     * The frame that was the source of the button press
     */
    sourceFrame: Frame;
};
type FrameRequest = FrameGETRequest | FramePOSTRequest;
type FrameStackBase = {
    timestamp: Date;
    /** speed in seconds */
    speed: number;
    responseStatus: number;
};
type FrameStackPending = {
    timestamp: Date;
    status: "pending";
} & FrameRequest;
type GetFrameResult = ReturnType<typeof getFrame>;
type FrameStackDone = FrameStackBase & FrameRequest & {
    frame: GetFrameResult;
    status: "done";
};
type FrameStackRequestError = FrameStackBase & FrameRequest & {
    status: "requestError";
    requestError: unknown;
};
type FrameStackMessage = FrameStackBase & FramePOSTRequest & {
    status: "message";
    message: string;
    type: "info" | "error";
};
type FramesStackItem = FrameStackPending | FrameStackDone | FrameStackRequestError | FrameStackMessage;
type FramesStack = FramesStackItem[];
type FrameReducerActions = {
    action: "LOAD";
    item: FrameStackPending;
} | {
    action: "REQUEST_ERROR";
    pendingItem: FrameStackPending;
    item: FrameStackRequestError;
} | {
    action: "DONE";
    pendingItem: FrameStackPending;
    item: FramesStackItem;
} | {
    action: "CLEAR";
} | {
    action: "RESET_INITIAL_FRAME";
    resultOrFrame: ParseResult | Frame;
    homeframeUrl: string | null | undefined;
};
type FrameState = {
    fetchFrame: (request: FrameRequest, 
    /**
     * If true, the frame stack will be cleared before the new frame is loaded
     *
     * @defaultValue false
     */
    shouldClear?: boolean) => Promise<void>;
    clearFrameStack: () => void;
    dispatchFrameStack: Dispatch<FrameReducerActions>;
    /** The frame at the top of the stack (at index 0) */
    frame: FramesStackItem | undefined;
    /** A stack of frames with additional context, with the most recent frame at index 0 */
    framesStack: FramesStack;
    inputText: string;
    setInputText: (s: string) => void;
    onButtonPress: (frame: Frame, frameButton: FrameButton, index: number, fetchFrameOverride?: (request: FrameRequest) => Promise<void>) => void | Promise<void>;
    homeframeUrl: string | null | undefined;
};
type OnMintArgs = {
    target: string;
    frameButton: FrameButton;
    frame: Frame;
};
type OnTransactionArgs = {
    transactionData: TransactionTargetResponse;
    frameButton: FrameButton;
    frame: Frame;
};
declare const themeParams: readonly ["bg", "buttonColor", "buttonBg", "buttonBorderColor", "buttonRadius", "buttonHoverBg"];
type FrameTheme = Partial<Record<(typeof themeParams)[number], string>>;
type FrameActionBodyPayload = Record<string, unknown>;
type FrameContext = Record<string, unknown>;

export { type FarcasterFrameContext as F, type GetFrameResult as G, type OnTransactionFunc as O, type SignerStateInstance as S, type UseFrameReturn as U, type FarcasterSignerState as a, type FarcasterSignerPendingApproval as b, createFrameActionMessageWithSignerKey as c, type FarcasterSignerImpersonating as d, type FarcasterSignerApproved as e, type FarcasterSigner as f, type FrameRequest as g, type FrameStackBase as h, type FrameStackPending as i, type FrameStackDone as j, type FrameStackRequestError as k, type FrameStackMessage as l, mockFarcasterSigner as m, type FramesStackItem as n, type FramesStack as o, type FrameReducerActions as p, type FrameState as q, type OnMintArgs as r, signFrameAction as s, type OnTransactionArgs as t, themeParams as u, type FrameTheme as v, type FrameActionBodyPayload as w, type FrameContext as x };
